"use strict";
// Copyright (c) Vidar Tonaas Fauske.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorBarView = exports.ColorBarModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const chromabar_1 = require("chromabar");
const d3_selection_1 = require("d3-selection");
const version_1 = require("../version");
class ColorBarModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ColorBarModel.model_name, _model_module: ColorBarModel.model_module, _model_module_version: ColorBarModel.model_module_version, _view_name: ColorBarModel.view_name, _view_module: ColorBarModel.view_module, _view_module_version: ColorBarModel.view_module_version, colormap: null, orientation: 'vertical', side: 'bottomright', length: 100, breadth: 30, border_thickness: 1, title: null, padding: 5, title_padding: 0, axis_padding: 0 });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.setupListeners();
    }
    setupListeners() {
        // register listener for current child value
        const childAttrName = 'colormap';
        var curValue = this.get(childAttrName);
        if (curValue) {
            this.listenTo(curValue, 'change', this.onChildChanged.bind(this));
            this.listenTo(curValue, 'childchange', this.onChildChanged.bind(this));
        }
        // make sure to (un)hook listeners when child points to new object
        this.on(`change:${childAttrName}`, (model, value) => {
            var prevModel = this.previous(childAttrName);
            var currModel = value;
            if (prevModel) {
                this.stopListening(prevModel);
            }
            if (currModel) {
                this.listenTo(currModel, 'change', this.onChildChanged.bind(this));
                this.listenTo(currModel, 'childchange', this.onChildChanged.bind(this));
            }
        }, this);
    }
    onChildChanged(model) {
        // Propagate up hierarchy:
        this.trigger('childchange', this);
    }
}
exports.ColorBarModel = ColorBarModel;
ColorBarModel.serializers = Object.assign(Object.assign({}, base_1.DOMWidgetModel.serializers), { colormap: { deserialize: base_1.unpack_models } });
ColorBarModel.model_name = 'ColorBarModel';
ColorBarModel.model_module = version_1.MODULE_NAME;
ColorBarModel.model_module_version = version_1.MODULE_VERSION;
ColorBarModel.view_name = 'ColorBarView';
ColorBarModel.view_module = version_1.MODULE_NAME;
ColorBarModel.view_module_version = version_1.MODULE_VERSION;
class ColorBarView extends base_1.DOMWidgetView {
    render() {
        const cmModel = this.model.get('colormap');
        this.barFunc = (0, chromabar_1.chromabar)(cmModel.obj);
        this.onChange();
        this.model.on('change', this.onChange, this);
        this.model.on('childchange', this.onChange, this);
    }
    _processLuminoMessage(msg, _super) {
        _super.call(this, msg);
        switch (msg.type) {
            case 'after-attach':
                // Auto-sizing should be updated when attached to DOM:
                this.onChange();
                break;
        }
    }
    processPhosphorMessage(msg) {
        this._processLuminoMessage(msg, base_1.DOMWidgetView.processPhosphorMessage);
    }
    processLuminoMessage(msg) {
        this._processLuminoMessage(msg, base_1.DOMWidgetView.processLuminoMessage);
    }
    onChange() {
        // Sync config:
        this.barFunc
            .orientation(this.model.get('orientation'))
            .side(this.model.get('side'))
            .barLength(this.model.get('length'))
            .breadth(this.model.get('breadth'))
            .borderThickness(this.model.get('border_thickness'))
            .title(this.model.get('title'))
            .padding(this.model.get('padding'))
            .titlePadding(this.model.get('title_padding'))
            .axisPadding(this.model.get('axis_padding'));
        // Update DOM:
        let svg = (0, d3_selection_1.select)(this.el)
            .selectAll('svg.jupyterColorbar').data([null]);
        svg = svg.merge(svg.enter().append('svg')
            .attr('class', 'jupyterColorbar'))
            .call(this.barFunc);
    }
}
exports.ColorBarView = ColorBarView;
//# sourceMappingURL=colorbar.js.map