/// <reference types="backbone" />
declare module "d3-scale" {
    function scaleDiverging<Output>(interpolator: ((t: number) => Output)): ScaleSequential<Output>;
}
import { TypedArray } from 'jupyter-dataserializers';
import { LinearScaleModel, LogScaleModel } from '../continuous';
import { SequentialScaleModel, OrdinalScaleModel } from '../scale';
/**
 * Contiguous color map.
 */
export declare class LinearColorScaleModel extends LinearScaleModel {
    defaults(): any;
    isColorScale: boolean;
}
/**
 * Contiguous color map.
 */
export declare class LogColorScaleModel extends LogScaleModel {
    defaults(): any;
    isColorScale: boolean;
    static model_name: string;
}
export declare class ArrayColorScaleModel extends SequentialScaleModel<string> {
    isColorScale: boolean;
    defaults(): any;
    createInterpolator(): (t: number) => any;
    syncToObject(): void;
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject(): any;
    protected colorInterp: ((t: number) => string) | null;
    static model_name: string;
    static serializers: {
        colors: {
            deserialize: typeof import("jupyter-dataserializers").JSONToUnionArray;
            serialize: typeof import("jupyter-dataserializers").unionToJSON;
        };
    };
}
export declare type ColorInterpolator = (t: number) => string;
/**
 * A contiguous color map created from a named color map.
 */
declare class NamedSequentialColorMapBase extends SequentialScaleModel<string> {
    getInterpolatorFactory(): ColorInterpolator;
    getInterpolatorFactoryName(): string | null;
    constructObject(): import("d3-scale").ScaleSequential<string>;
    /**
     * Sync the model properties to the d3 object.
     */
    syncToObject(): void;
    syncToModel(toSet: Backbone.ObjectHash): void;
    isColorScale: boolean;
}
/**
 * A contiguous color map created from a named color map.
 */
export declare class NamedSequentialColorMap extends NamedSequentialColorMapBase {
    defaults(): any;
    static model_name: string;
}
/**
 * A contiguous color map created from a named color map.
 */
export declare class NamedDivergingColorMap extends NamedSequentialColorMapBase {
    defaults(): any;
    constructObject(): import("d3-scale").ScaleSequential<string>;
    static model_name: string;
}
/**
 * A contiguous color map created from a named color map.
 */
export declare class NamedOrdinalColorMap extends OrdinalScaleModel {
    defaults(): any;
    createPropertiesArrays(): void;
    getScheme(): string[];
    getSchemeName(): string | null;
    constructObject(): import("d3-scale").ScaleOrdinal<string, string>;
    /**
     * Sync the model properties to the d3 object.
     */
    syncToObject(): void;
    syncToModel(toSet: Backbone.ObjectHash): void;
    isColorScale: boolean;
    static model_name: string;
}
export interface ColorScale {
    copy(): this;
    domain(domain: number[]): this;
    (value: number | {
        valueOf(): number;
    }): string;
}
export interface ColorMapModel {
    obj: ColorScale;
    isColorScale: true;
}
export declare function isColorMapModel(candidate: any): candidate is ColorMapModel;
export declare function colormapAsRGBArray(mapModel: ColorMapModel, size: number): Uint8ClampedArray;
export declare function colormapAsRGBArray<T extends TypedArray>(mapModel: ColorMapModel, array: T): T;
export declare function colormapAsRGBAArray(mapModel: ColorMapModel, size: number): Uint8ClampedArray;
export declare function colormapAsRGBAArray<T extends TypedArray>(mapModel: ColorMapModel, array: T): T;
export {};
