"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.colormapAsRGBAArray = exports.colormapAsRGBArray = exports.isColorMapModel = exports.NamedOrdinalColorMap = exports.NamedDivergingColorMap = exports.NamedSequentialColorMap = exports.ArrayColorScaleModel = exports.LogColorScaleModel = exports.LinearColorScaleModel = void 0;
const d3_color_1 = require("d3-color");
const d3_interpolate_1 = require("d3-interpolate");
const d3_scale_1 = require("d3-scale");
const d3Chromatic = __importStar(require("d3-scale-chromatic"));
const jupyter_dataserializers_1 = require("jupyter-dataserializers");
const ndarray = require("ndarray");
const continuous_1 = require("../continuous");
const scale_1 = require("../scale");
const utils_1 = require("../utils");
/**
 * Contiguous color map.
 */
class LinearColorScaleModel extends continuous_1.LinearScaleModel {
    constructor() {
        super(...arguments);
        this.isColorScale = true;
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { range: ['black', 'white'] });
    }
}
exports.LinearColorScaleModel = LinearColorScaleModel;
/**
 * Contiguous color map.
 */
class LogColorScaleModel extends continuous_1.LogScaleModel {
    constructor() {
        super(...arguments);
        this.isColorScale = true;
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { range: ['black', 'white'] });
    }
}
exports.LogColorScaleModel = LogColorScaleModel;
LogColorScaleModel.model_name = 'LogColorScaleModel';
class ArrayColorScaleModel extends scale_1.SequentialScaleModel {
    constructor() {
        super(...arguments);
        this.isColorScale = true;
        this.colorInterp = null;
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { colors: ndarray(new Float32Array([0, 0, 0, 1, 1, 1]), [2, 3]), space: 'rgb', gamma: 1.0 });
    }
    createInterpolator() {
        const space = this.get('space');
        const colors = this.get('colors');
        const factory = space === 'hsl' ? d3_color_1.hsl : d3_color_1.rgb;
        const spaceColors = [];
        const alpha = colors.shape[1] > 3;
        if (space === 'hsl') {
            for (let i = 0; i < colors.shape[0]; ++i) {
                spaceColors.push(factory(360 * colors.get(i, 0), colors.get(i, 1), colors.get(i, 2), alpha ? colors.get(i, 3) : 1.0));
            }
            return (0, d3_interpolate_1.piecewise)(d3_interpolate_1.interpolateHsl, spaceColors);
        }
        for (let i = 0; i < colors.shape[0]; ++i) {
            spaceColors.push(factory(255 * colors.get(i, 0), 255 * colors.get(i, 1), 255 * colors.get(i, 2), alpha ? colors.get(i, 3) : 1.0));
        }
        let gamma = this.get('gamma');
        if (gamma === undefined || gamma === null) {
            gamma = 1.0;
        }
        return (0, d3_interpolate_1.piecewise)(d3_interpolate_1.interpolateRgb.gamma(gamma), spaceColors);
    }
    syncToObject() {
        super.syncToObject();
        const interpProps = ['colors', 'space', 'gamma'];
        const interpChange = interpProps.some(prop => this.hasChanged(prop));
        if (interpChange) {
            this.obj.interpolator(this.createInterpolator());
        }
    }
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject() {
        return (0, d3_scale_1.scaleSequential)(this.createInterpolator());
    }
}
exports.ArrayColorScaleModel = ArrayColorScaleModel;
ArrayColorScaleModel.model_name = 'ArrayColorScaleModel';
ArrayColorScaleModel.serializers = Object.assign(Object.assign({}, scale_1.SequentialScaleModel.serializers), { colors: jupyter_dataserializers_1.data_union_array_serialization });
const chromaticInterpLut = {};
for (let key of Object.keys(d3Chromatic)) {
    if (key.indexOf('interpolate') === 0) {
        const lowKey = key.slice('interpolate'.length).toLowerCase();
        chromaticInterpLut[lowKey] = d3Chromatic[key];
    }
}
const chromaticSchemeLut = {};
for (let key of Object.keys(d3Chromatic)) {
    if (key.indexOf('scheme') === 0) {
        const lowKey = key.slice('scheme'.length).toLowerCase();
        chromaticSchemeLut[lowKey] = d3Chromatic[key];
    }
}
function isFixedScheme(candidate) {
    return candidate.length < 4 || !Array.isArray(candidate[3]);
}
/**
 * A contiguous color map created from a named color map.
 */
class NamedSequentialColorMapBase extends scale_1.SequentialScaleModel {
    constructor() {
        super(...arguments);
        this.isColorScale = true;
    }
    getInterpolatorFactory() {
        let name = this.get('name');
        return chromaticInterpLut[name.toLowerCase()];
    }
    getInterpolatorFactoryName() {
        const interp = this.obj.interpolator();
        // Do a reverse lookup in d3Chromatic
        const lut = d3Chromatic;
        for (let key of Object.keys(lut)) {
            if (interp === lut[key]) {
                const name = key.replace(/^interpolate/, '');
                return name;
            }
        }
        throw new Error(`Unknown color interpolator name of function: ${interp}`);
    }
    constructObject() {
        const interpolator = this.getInterpolatorFactory();
        return (0, d3_scale_1.scaleSequential)(interpolator);
    }
    /**
     * Sync the model properties to the d3 object.
     */
    syncToObject() {
        super.syncToObject();
        const interpolator = this.getInterpolatorFactory();
        this.obj
            .interpolator(interpolator);
    }
    syncToModel(toSet) {
        toSet['name'] = this.getInterpolatorFactoryName();
        super.syncToModel(toSet);
    }
}
/**
 * A contiguous color map created from a named color map.
 */
class NamedSequentialColorMap extends NamedSequentialColorMapBase {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { name: 'Viridis' });
    }
}
exports.NamedSequentialColorMap = NamedSequentialColorMap;
NamedSequentialColorMap.model_name = 'NamedSequentialColorMap';
/**
 * A contiguous color map created from a named color map.
 */
class NamedDivergingColorMap extends NamedSequentialColorMapBase {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { name: 'BrBG', domain: [0, 0.5, 1] });
    }
    constructObject() {
        const interpolator = this.getInterpolatorFactory();
        return (0, d3_scale_1.scaleDiverging)(interpolator);
    }
}
exports.NamedDivergingColorMap = NamedDivergingColorMap;
NamedDivergingColorMap.model_name = 'NamedDivergingColorMap';
/**
 * A contiguous color map created from a named color map.
 */
class NamedOrdinalColorMap extends scale_1.OrdinalScaleModel {
    constructor() {
        super(...arguments);
        this.isColorScale = true;
    }
    defaults() {
        const def = Object.assign(Object.assign({}, super.defaults()), { name: 'Category10', cardinality: 10 });
        delete def.range;
        return def;
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.splice(this.simpleProperties.indexOf('range'), 1);
    }
    getScheme() {
        const name = this.get('name');
        const scheme = chromaticSchemeLut[name.toLowerCase()];
        if (!scheme) {
            throw new Error(`Unknown scheme name: ${name}`);
        }
        if (isFixedScheme(scheme)) {
            return scheme;
        }
        const cardinality = this.get('cardinality');
        return scheme[cardinality];
    }
    getSchemeName() {
        const scheme = this.obj.range();
        // Do a reverse lookup in d3Chromatic
        const lut = d3Chromatic;
        for (let key of Object.keys(lut)) {
            let candidate = lut[key];
            if (!candidate || !Array.isArray(candidate)) {
                continue;
            }
            if (!isFixedScheme(candidate)) {
                candidate = candidate[scheme.length];
                if (!candidate) {
                    continue;
                }
            }
            if ((0, utils_1.arrayEquals)(scheme, candidate)) {
                const name = key.replace(/^scheme/, '');
                return name;
            }
        }
        throw new Error(`Unknown color scheme name for range: ${scheme}`);
    }
    constructObject() {
        const scheme = this.getScheme();
        return (0, d3_scale_1.scaleOrdinal)(scheme);
    }
    /**
     * Sync the model properties to the d3 object.
     */
    syncToObject() {
        super.syncToObject();
        const scheme = this.getScheme();
        this.obj
            .range(scheme);
    }
    syncToModel(toSet) {
        toSet['name'] = this.getSchemeName();
        super.syncToModel(toSet);
    }
}
exports.NamedOrdinalColorMap = NamedOrdinalColorMap;
NamedOrdinalColorMap.model_name = 'NamedOrdinalColorMap';
function isColorMapModel(candidate) {
    return (candidate !== null && candidate !== undefined &&
        candidate.isColorScale === true);
}
exports.isColorMapModel = isColorMapModel;
function colormapAsRGBArray(mapModel, data) {
    let n;
    if (typeof data === 'number') {
        n = data;
        data = new Uint8ClampedArray(n * 3);
    }
    else {
        n = data.length / 3;
    }
    const scale = mapModel.obj.copy().domain([0, n]);
    for (let i = 0; i < n; ++i) {
        const color = (0, d3_color_1.rgb)(scale(i));
        data[i * 3 + 0] = color.r;
        data[i * 3 + 1] = color.g;
        data[i * 3 + 2] = color.b;
    }
    return data;
}
exports.colormapAsRGBArray = colormapAsRGBArray;
function colormapAsRGBAArray(mapModel, data) {
    let n;
    if (typeof data === 'number') {
        n = data;
        data = new Uint8ClampedArray(n * 4);
    }
    else {
        n = data.length / 4;
    }
    let scale;
    let values = Array.from(new Array(n), (x, i) => i); // range(n)
    if (mapModel instanceof NamedDivergingColorMap) {
        scale = mapModel.obj.copy().domain([0, n / 2, n]);
    }
    else if (mapModel instanceof scale_1.OrdinalScaleModel) {
        scale = mapModel.obj;
        values = scale.domain();
    }
    else {
        scale = mapModel.obj.copy().domain([0, n - 1]);
    }
    for (let i of values) {
        const color = (0, d3_color_1.rgb)(scale(i));
        data[i * 4 + 0] = color.r;
        data[i * 4 + 1] = color.g;
        data[i * 4 + 2] = color.b;
        data[i * 4 + 3] = 255 * color.opacity;
    }
    return data;
}
exports.colormapAsRGBAArray = colormapAsRGBAArray;
//# sourceMappingURL=scales.js.map