/// <reference types="backbone" />
import { ScaleLinear } from 'd3-scale';
import { ScaleModel } from './scale';
/**
 * A widget model of a continuous scale
 */
export declare abstract class ContinuousScaleModel extends ScaleModel {
    defaults(): {
        domain: number[];
        range: number[];
        interpolator: string;
        clamp: boolean;
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    abstract constructObject(): any;
    /**
     * Sync the model properties to the d3 object.
     */
    syncToObject(): void;
    /**
     * Synt the d3 object properties to the model.
     */
    syncToModel(toSet: Backbone.ObjectHash): void;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: import("@jupyter-widgets/base").IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: import("@jupyter-widgets/base").WidgetModel | undefined) => any) | undefined;
        };
    };
}
/**
 * A widget model of a linear scale
 */
export declare class LinearScaleModel extends ContinuousScaleModel {
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject(): any;
    obj: ScaleLinear<any, any>;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: import("@jupyter-widgets/base").IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: import("@jupyter-widgets/base").WidgetModel | undefined) => any) | undefined;
        };
    };
    static model_name: string;
}
/**
 * A widget model of a linear scale
 */
export declare class LogScaleModel extends ContinuousScaleModel {
    defaults(): {
        base: number;
        domain: number[];
        range: number[];
        interpolator: string;
        clamp: boolean;
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject(): any;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: import("@jupyter-widgets/base").IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: import("@jupyter-widgets/base").WidgetModel | undefined) => any) | undefined;
        };
    };
    static model_name: string;
}
/**
 * A widget model of a linear scale
 */
export declare class PowScaleModel extends ContinuousScaleModel {
    defaults(): {
        exponent: number;
        domain: number[];
        range: number[];
        interpolator: string;
        clamp: boolean;
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject(): any;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: import("@jupyter-widgets/base").IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: import("@jupyter-widgets/base").WidgetModel | undefined) => any) | undefined;
        };
    };
    static model_name: string;
}
