"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PowScaleModel = exports.LogScaleModel = exports.LinearScaleModel = exports.ContinuousScaleModel = void 0;
const d3_scale_1 = require("d3-scale");
const d3Interpolate = __importStar(require("d3-interpolate"));
const scale_1 = require("./scale");
/**
 * Find the name of the d3-interpolate function
 */
function interpolatorName(fun) {
    for (let key of Object.keys(d3Interpolate)) {
        if (d3Interpolate[key] === fun) {
            return key;
        }
    }
    throw ReferenceError(`Cannot find name of interpolator ${fun}`);
}
/**
 * A widget model of a continuous scale
 */
class ContinuousScaleModel extends scale_1.ScaleModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { domain: [0, 1], range: [0, 1], interpolator: 'interpolate', clamp: false });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('domain', 'range', 'clamp');
    }
    /**
     * Sync the model properties to the d3 object.
     */
    syncToObject() {
        super.syncToObject();
        let interpolatorName = this.get('interpolator') || 'interpolate';
        let interpolator = d3Interpolate[interpolatorName];
        this.obj.interpolate(interpolator);
    }
    /**
     * Synt the d3 object properties to the model.
     */
    syncToModel(toSet) {
        let interpolator = this.obj.interpolate();
        toSet['interpolator'] = interpolatorName(interpolator);
        super.syncToModel(toSet);
    }
}
exports.ContinuousScaleModel = ContinuousScaleModel;
ContinuousScaleModel.serializers = Object.assign({}, scale_1.ScaleModel.serializers);
/**
 * A widget model of a linear scale
 */
class LinearScaleModel extends ContinuousScaleModel {
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject() {
        return (0, d3_scale_1.scaleLinear)();
    }
}
exports.LinearScaleModel = LinearScaleModel;
LinearScaleModel.serializers = Object.assign({}, ContinuousScaleModel.serializers);
LinearScaleModel.model_name = 'LinearScaleModel';
/**
 * A widget model of a linear scale
 */
class LogScaleModel extends ContinuousScaleModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { base: 10, domain: [1, 10] });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('base');
    }
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject() {
        return (0, d3_scale_1.scaleLog)();
    }
}
exports.LogScaleModel = LogScaleModel;
LogScaleModel.serializers = Object.assign({}, ContinuousScaleModel.serializers);
LogScaleModel.model_name = 'LogScaleModel';
/**
 * A widget model of a linear scale
 */
class PowScaleModel extends ContinuousScaleModel {
    defaults() {
        this.constructor;
        return Object.assign(Object.assign({}, super.defaults()), { exponent: 1 });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('exponent');
    }
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    constructObject() {
        return (0, d3_scale_1.scalePow)();
    }
}
exports.PowScaleModel = PowScaleModel;
PowScaleModel.serializers = Object.assign({}, ContinuousScaleModel.serializers);
PowScaleModel.model_name = 'PowScaleModel';
//# sourceMappingURL=continuous.js.map