"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.DropdownView = exports.WidgetDropdownModel = exports.StringDropdownModel = exports.SelectorBaseModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
const utils_1 = require("./utils");
/**
 * Base model for scales
 */
class SelectorBaseModel extends base_1.DOMWidgetModel {
    defaults() {
        const ctor = this.constructor;
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ctor.model_name, _model_module: ctor.model_module, _model_module_version: ctor.model_module_version, _view_name: ctor.view_name, _view_module: ctor.view_module, _view_module_version: ctor.view_module_version });
    }
}
exports.SelectorBaseModel = SelectorBaseModel;
SelectorBaseModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
SelectorBaseModel.model_module = version_1.MODULE_NAME;
SelectorBaseModel.model_module_version = version_1.MODULE_VERSION;
SelectorBaseModel.view_name = null;
SelectorBaseModel.view_module = version_1.MODULE_NAME;
SelectorBaseModel.view_module_version = version_1.MODULE_VERSION;
/**
 * A widget model of a linear scale
 */
class StringDropdownModel extends SelectorBaseModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { value: null, options: [] });
    }
    getLabels() {
        return this.get('options') || [];
    }
    ;
    get selectedLabel() {
        return this.get('value');
    }
    set selectedLabel(value) {
        this.set({ value }, 'fromView');
        this.save_changes();
    }
}
exports.StringDropdownModel = StringDropdownModel;
StringDropdownModel.serializers = Object.assign({}, SelectorBaseModel.serializers);
StringDropdownModel.model_name = 'StringDropdownModel';
/**
 * A widget model of a linear scale
 */
class WidgetDropdownModel extends SelectorBaseModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { value: null, options: {} });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.setupListeners();
        this.updateReverse(this, this.get('options') || []);
    }
    setupListeners() {
        this.on('change:options', this.updateReverse, this);
    }
    updateReverse(model, value, options) {
        this.reverseMap = {};
        for (let key of Object.keys(value)) {
            const v = value[key];
            if (v && v.toJSON) {
                this.reverseMap[value[key].toJSON({})] = key;
            }
        }
    }
    getLabels() {
        const options = this.get('options') || {};
        return Object.keys(options);
    }
    ;
    get selectedLabel() {
        const value = this.get('value');
        return value && this.reverseMap[value.toJSON(undefined)];
    }
    set selectedLabel(value) {
        const options = this.get('options') || {};
        this.set({ value: value ? options[value] || null : null }, 'fromView');
        this.save_changes();
    }
}
exports.WidgetDropdownModel = WidgetDropdownModel;
WidgetDropdownModel.serializers = Object.assign(Object.assign({}, SelectorBaseModel.serializers), { value: { deserialize: base_1.unpack_models }, options: { deserialize: base_1.unpack_models } });
WidgetDropdownModel.model_name = 'StringDropdownModel';
class DropdownView extends base_1.DOMWidgetView {
    constructor() {
        super(...arguments);
        this._current_options = [];
    }
    /**
     * Public constructor.
     */
    initialize(parameters) {
        super.initialize(parameters);
        this.listenTo(this.model, 'change', this.onModelChange.bind(this));
    }
    onModelChange(model, options) {
        if (options === 'fromView') {
            return;
        }
        this.update();
    }
    /**
     * Called when view is rendered.
     */
    render() {
        super.render();
        this.el.classList.add('jupyter-widgets');
        this.el.classList.add('widget-inline-hbox');
        this.el.classList.add('widget-dropdown');
        this.listbox = document.createElement('select');
        this.el.appendChild(this.listbox);
        this.update();
    }
    /**
     * Update the contents of this view
     */
    update() {
        const labels = this.model.getLabels();
        if (!(0, utils_1.arrayEquals)(labels, this._current_options)) {
            this._updateOptions(labels);
        }
        // Select the correct element
        const sel = this.model.selectedLabel;
        if (sel === null) {
            this.listbox.selectedIndex = -1;
        }
        else {
            this.listbox.value = sel;
        }
        return super.update();
    }
    _updateOptions(labels) {
        this.listbox.textContent = '';
        for (let label of labels) {
            let option = document.createElement('option');
            option.textContent = label.replace(/ /g, '\xa0'); // space -> &nbsp;
            option.value = label;
            this.listbox.appendChild(option);
        }
        this._current_options = labels.slice();
    }
    events() {
        return {
            'change select': '_handle_change'
        };
    }
    /**
     * Handle when a new value is selected.
     */
    _handle_change() {
        this.model.selectedLabel = this.listbox.selectedIndex === -1
            ? null
            : this.listbox.value;
    }
}
exports.DropdownView = DropdownView;
//# sourceMappingURL=selectors.js.map